@@ -288,6 +288,17 @@ static int buf_write_convert(struct bw_info *ip, char **bufp, int *lenp)
         c = n > 1 ? (unsigned)utf_ptr2char(*bufp + wlen)
                   : (uint8_t)(*bufp)[wlen];
       }
+      // Check that there is enough space
+      if (!(flags & FIO_LATIN1)) {
+        size_t need = (flags & FIO_UCS4) ? 4 : 2;
+        if ((flags & FIO_UTF16) && c >= 0x10000) {
+          need = 4;
+        }
+
+        if ((size_t)(p - ip->bw_conv_buf) + need > ip->bw_conv_buflen) {
+          return FAIL;
+        }
+      }
 
       if (ucs2bytes(c, &p, flags) && !ip->bw_conv_error) {
         ip->bw_conv_error = true;
@@ -335,7 +346,7 @@ static int buf_write_bytes(struct bw_info *ip)
     // Only checking conversion, which is OK if we get here.
     return OK;
   }
-  int wlen = write_eintr(ip->bw_fd, buf, (size_t)len);
+  int wlen = (int)write_eintr(ip->bw_fd, buf, (size_t)len);
   return (wlen < len) ? FAIL : OK;
 }
 
@@ -1291,11 +1302,13 @@ int buf_write(buf_T *buf, char *fname, char *sfname, linenr_T start, linenr_T en
   if (converted) {
     wb_flags = get_fio_flags(fenc);
     if (wb_flags & (FIO_UCS2 | FIO_UCS4 | FIO_UTF16 | FIO_UTF8)) {
+      // overallocate a bit, in case we read incomplete multi-byte chars
+      int size = bufsize + CONV_RESTLEN;
       // Need to allocate a buffer to translate into.
       if (wb_flags & (FIO_UCS2 | FIO_UTF16 | FIO_UTF8)) {
-        write_info.bw_conv_buflen = (size_t)bufsize * 2;
+        write_info.bw_conv_buflen = (size_t)size * 2;
       } else {       // FIO_UCS4
-        write_info.bw_conv_buflen = (size_t)bufsize * 4;
+        write_info.bw_conv_buflen = (size_t)size * 4;
       }
       write_info.bw_conv_buf = verbose_try_malloc(write_info.bw_conv_buflen);
       if (!write_info.bw_conv_buf) {
