@@ -19,12 +19,11 @@ describe('default', function()
       n.command('let g:n=0')
       n.command('au BufEnter * let g:n = g:n + 1')
 
-      n.command('terminal')
-      t.eq(1, n.eval('get(g:, "n", 0)'))
+      t.eq(1, n.exec_lua('vim.cmd.terminal(); return vim.g.n'))
 
       t.retry(nil, 1000, function()
         t.neq('terminal', n.api.nvim_get_option_value('buftype', { buf = 0 }))
-        t.eq(2, n.eval('get(g:, "n", 0)'))
+        t.eq(2, n.eval('g:n'))
       end)
     end)
   end)
