@@ -3,10 +3,11 @@
 -- And test "ra" on multibyte characters.
 -- Also test byteidx() and byteidxcomp()
 
-local helpers = require('test.functional.helpers')(after_each)
-local feed, insert, eq, eval, clear, feed_command, expect = helpers.feed,
-  helpers.insert, helpers.eq, helpers.eval, helpers.clear, helpers.feed_command,
-  helpers.expect
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
+
+local feed, insert, eq, eval, clear, feed_command, expect =
+  n.feed, n.insert, t.eq, n.eval, n.clear, n.feed_command, n.expect
 
 describe('multibyte text', function()
   before_each(clear)
@@ -182,9 +183,8 @@ describe('multibyte text', function()
     feed_command('/^{/+1')
     feed_command('set noai tw=2 fo=tm')
     feed('gqgqjgqgqo<cr>')
-    -- Literal spaces will be trimmed from the by feed().
-    feed('<space><space>Ｘ<cr>')
-    feed('<space><space>Ｘa<esc>')
+    feed('  Ｘ<cr>')
+    feed('  Ｘa<esc>')
     expect([[
       {
         Ｘ
